/**
 * The Modal jQuery plugin
 *
 * @author Alexander Makarov <sam@rmcreative.ru>
 * @link https://github.com/samdark/the-modal
 * @version 1.0
 */
;(function($, window, document, undefined) {
	"use strict";
	/*jshint smarttabs:true*/

	var pluginNamespace = 'the-modal',
		// global defaults
    	defaults = {
			overlayClass: 'themodal-overlay',

			closeOnEsc: true,
			closeOnOverlayClick: true,
            closeOnButton: true,
			onClose: null,
			onOpen: null,
            onReposition: null,
            isReposition: false,
        };

	function lockContainer() {
		$('html,body').addClass('lock');
	}
	
	function unlockContainer() {
		$('html,body').removeClass('lock');
	}
    function repositionDialog(el, callback) {
        var percent = $(window).scrollTop() / ($(document).height() - $(window).height());
        var startPosition = percent * ($(document).height() - $(window).height());
        startPosition = startPosition > 0 ? startPosition : 0;
        var marginTop = ($(window).height() - $(el).height())/2;
        marginTop = marginTop > 0 ? marginTop : 0;
        $(el).css('margin-top', startPosition + marginTop);
        if(null != callback) {
            callback($(el));
        }
    }
	function init(els, options) {
		var modalOptions = options;

		if(els.length) {
			els.each(function(){
				$(this).data(pluginNamespace+'.options', modalOptions);
			});
		}
		else {
			$.extend(defaults, modalOptions);
		}

		return {
			open: function(options) {
				var el = els.get(0);
				var localOptions = $.extend({}, defaults, $(el).data(pluginNamespace+'.options'), options);

				// close modal if opened
				if($('.'+localOptions.overlayClass).length) {
					$.modal().close();
				}
				
				lockContainer();

				var overlay = $('<div/>').addClass(localOptions.overlayClass).prependTo('body');/*.css('height', $(document).height());*/
				overlay.data(pluginNamespace+'.options', options);

				if(el) {
					//$(el).clone(true).appendTo(overlay).show();
                    $(el).appendTo(overlay).show();
				}

				if(localOptions.closeOnEsc) {
					$(document).bind('keyup.'+pluginNamespace, function(e){
						if(e.keyCode === 27) {
							$.modal().close();
						}
					});
				}

				if(localOptions.closeOnOverlayClick) {
					overlay.children().on('click.' + pluginNamespace, function(e){
						e.stopPropagation();
					});
					$('.' + localOptions.overlayClass).on('click.' + pluginNamespace, function(e){
						$.modal().close();
					});
				}
                if(localOptions.isReposition) {
                    $(window).scroll(function(){
                        repositionDialog(el, localOptions.onReposition);
                	});
                    repositionDialog(el, localOptions.onReposition);
                }

				$(document).bind('touchmove.'+pluginNamespace,function(e){
					if(!$(e).parents('.' + localOptions.overlayClass)) {
						e.preventDefault();
					}
				});

				if(localOptions.onOpen) {
					localOptions.onOpen(overlay, localOptions);
				}
                if(localOptions.closeOnButton) {
      		        $('.modal .close').on('click', function(e){
            			e.preventDefault();
            			$.modal().close();
            		});
                }

			},
			close: function() {
				var el = els.get(0);

				var localOptions = $.extend({}, defaults, options);
				var overlay = $('.' + localOptions.overlayClass);
				$.extend(localOptions, overlay.data(pluginNamespace+'.options'));
                //
                overlay.children().hide().appendTo('body');
                //
				overlay.remove();
				unlockContainer();

				if(localOptions.closeOnEsc) {
					$(document).unbind('keyup.'+pluginNamespace);
				}

				if(localOptions.onClose) {
					localOptions.onClose(overlay, localOptions);
				}
			}
		};
	}

	$.modal = function(options){
		return init($(), options);
	};

	$.fn.modal = function(options) {
		return init(this, options);
	};

})(jQuery, window, document);
